package com.fjhx.admin.service.supplier.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.admin.entity.product.po.ProductInfo;
import com.fjhx.admin.entity.product.vo.SupplierPriceAddVo;
import com.fjhx.admin.entity.supplier.dto.SupplierPriceDto;
import com.fjhx.admin.entity.supplier.dto.SupplierPriceSelectDto;
import com.fjhx.admin.entity.supplier.po.SupplierInfo;
import com.fjhx.admin.entity.supplier.po.SupplierPrice;
import com.fjhx.admin.entity.supplier.vo.SupplierPriceVo;
import com.fjhx.admin.mapper.supplier.SupplierPriceMapper;
import com.fjhx.admin.service.supplier.SupplierPriceService;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * <p>
 * 供应商价格 服务实现类
 * </p>
 *
 * @author zlj
 * @since 2023-03-17
 */
@Service
public class SupplierPriceServiceImpl extends ServiceImpl<SupplierPriceMapper, SupplierPrice> implements SupplierPriceService {

    @Override
    public Page<SupplierPriceVo> getPage(SupplierPriceSelectDto dto) {
        IWrapper<SupplierPrice> wrapper = getWrapper();
        wrapper.orderByDesc("sp", SupplierPrice::getId);
        wrapper.eq("si", SupplierInfo::getType, dto.getSupplierType());
        wrapper.eq("pi", ProductInfo::getDefinition, dto.getDefinition());
        wrapper.keyword(dto,
                new SqlField("si.name"),
                new SqlField("pi.name"),
                new SqlField("pi.code")
        );
        Page<SupplierPriceVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public SupplierPriceAddVo add(SupplierPriceDto supplierPriceDto) {
        SupplierPriceAddVo supplierPriceAddVo = new SupplierPriceAddVo();

        // 添加产品列表
        List<SupplierPrice> supplierPriceList = supplierPriceDto.getSupplierPriceList();
        // 添加产品id列表
        List<Long> productInfoIdList = supplierPriceList.stream().map(SupplierPrice::getProductInfoId).collect(Collectors.toList());

        // 供应商id
        Long supplierInfoId = supplierPriceDto.getSupplierInfoId();

        // 存在供应商价格列表
        List<SupplierPrice> existSupplierPriceList = list(q -> q
                .eq(SupplierPrice::getSupplierInfoId, supplierInfoId)
                .in(SupplierPrice::getProductInfoId, productInfoIdList));

        if (existSupplierPriceList.size() == 0) {
            supplierPriceList.forEach(item -> item.setSupplierInfoId(supplierPriceDto.getSupplierInfoId()));
            saveBatch(supplierPriceList);
            supplierPriceAddVo.setAddSuccess(true);
            return supplierPriceAddVo;
        }

        // 存在价格更新
        if (supplierPriceDto.isRenewWhileItExists()) {

            // 产品id,明细map
            Map<Long, SupplierPrice> map = existSupplierPriceList.stream()
                    .collect(Collectors.toMap(SupplierPrice::getProductInfoId, Function.identity()));

            for (SupplierPrice supplierPrice : supplierPriceList) {
                supplierPrice.setSupplierInfoId(supplierInfoId);

                SupplierPrice existSupplierPrice = map.get(supplierPrice.getProductInfoId());
                if (existSupplierPrice != null) {
                    supplierPrice.setId(existSupplierPrice.getId());
                    supplierPrice.setPrice(existSupplierPrice.getPrice());
                }
            }
            saveOrUpdateBatch(supplierPriceList);
            supplierPriceAddVo.setAddSuccess(true);
            return supplierPriceAddVo;
        }

        // 存在价格不更新
        supplierPriceAddVo.setExistSupplierPriceList(existSupplierPriceList);
        return supplierPriceAddVo;
    }

    @Override
    public void edit(SupplierPriceDto supplierPriceDto) {
        supplierPriceDto.setSupplierInfoId(null);
        supplierPriceDto.setProductInfoId(null);

        this.updateById(supplierPriceDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

}
